/**
 * This callback is called on a socket.on "corpus_analysis_send_meta_data".
 * Handels incoming corpus metadata
 */
function saveMetaData(...args) {
  let [payload, client, results, rest] = args;
  client.notifyView('meta-data-recieving');
  results.metaData.init(payload)
  console.info('Metada saved:', results);
  client.isBusy = false;
  client.notifyView('meta-data-recieved');
}

/**
 * This callback should be registered to the SocketEventListener 'recieveQueryStatus'.
 * It just gets the incoming status data of the issued query
 * and does some preperation work like hiding or showing elements and deleting
 * the data from the last query.
 */
function prepareQueryData(...args) {
  // deletes old data from query issued before this new query
  let [payload, client, results, rest] = args;
  // always initialize the results to delete data from the query issued before
  results.init();
  results.data.match_count = payload.match_count;
  client.requestQueryProgress = 0;
  client.notifyView('query-data-prepareing', { results: results });
}

/**
 * This callbacks saves the incoming query data chunks into the model results.
 */
function saveQueryData(...args) {
  let [payload, client, results, rest] = args;
  // Get data matches length before new chunk data is being inserted
  let dataLength = results.data.matches.length;
  if (client.dynamicMode) {
    // Incorporating new chunk data into full results
    results.data.matches.push(...payload.chunk.matches);
    results.data.addData(payload.chunk.cpos_lookup, 'cpos_lookup');
    results.data.addData(payload.chunk.text_lookup, 'text_lookup');
     /**
      * Increment match_counts per text in a global results varaible because
      * they are coming in chunkwise.
      */
    if (payload.chunk.text_lookup) {
      for (let [text_key, value] of Object.entries(payload.chunk.text_lookup)) {
        if (!(text_key in results.tmp_match_counts)) {
          results.tmp_match_counts[text_key] = {match_count: 0};
        }
        results.tmp_match_counts[text_key].match_count += payload.chunk.text_lookup[text_key].match_count;
      }
    }
    results.data.cpos_ranges = payload.chunk.cpos_ranges;
    let queryFormElement = document.querySelector('#query-form');
    results.data.getQueryStr(queryFormElement);
    client.requestQueryProgress = payload.progress;
    client.notifyView('query-data-recieving',
                      { results: results,
                        client: client,
                        dataLength: dataLength });
    console.info('Query data chunk saved', results.data);
    if (client.requestQueryProgress === 100) {
      client.isBusy = false;
      // Update text_lookup with tmp_match_counts.
      for (let [text_key, value] of Object.entries(results.tmp_match_counts)) {
        results.data.text_lookup[text_key].match_count = results.tmp_match_counts[text_key].match_count;
      }
      client.notifyView('query-data-recieved');
    }
  } else {
    results.data.matches.push(...payload.matches);
    results.data.addData(payload.cpos_lookup, 'cpos_lookup');
    results.data.addData(payload.text_lookup, 'text_lookup');
    results.data.cpos_ranges = payload.cpos_ranges;
    let queryFormElement = document.querySelector('#query-form');
    results.data.getQueryStr(queryFormElement);
    client.requestQueryProgress = 100;
    client.notifyView('query-data-recieving',
                      { results: results,
                        client: client,
                        dataLength: dataLength });
    console.info('Query data chunk saved', results.data);
    if (client.requestQueryProgress === 100) {
      console.log(results.data);
      client.notifyView('query-data-recieved');
    }
  }
}

/**
 * This callback gets the results data for the export. Either requesting it
 * whith full context from the server or gets it locally without full context
 * from the already present results.data. Result data is identified with the
 * dataIndexes. On index is one match.
 */
function getResultsData(...args) {
  let [resultsType, dataIndexes, resultsList, client, results, rest] = args;
  client.isBusy = true;
  if (resultsList.exportFullInspectContext.checked
      || resultsType === 'inspect-results') {
    console.info('Get results with full context');
    client.getResultsData(resultsType, dataIndexes, results);
  } else {
    console.info('Get results without full context');
    client.getResultsDataWithoutContext(resultsType, dataIndexes, results,
                                        resultsList);
  }
}

/**
 * Handles incoming results which have been requested via getResultsData(). and
 * saves the data accorindgly into the results object.
 */
function saveResultsData(...args) {
  let [payload, type, client, results, rest] = args;
  let objectKey = '';
  if (type === 'full-results') {
    console.info('Saving full-results data.');
    objectKey = 'fullResultsData';
  } else if (type === 'sub-results') {
    console.info('Saving sub-results data.');
    objectKey = 'subResultsData';
  } else if (type = 'inspect-results') {
    objectKey = 'inspectResultsData';
    console.info('Saving inspect-results data');
  }
  // Save incoming data. Data is incoming one match at a time.
  results[objectKey].matches.push(...payload.matches);
  results[objectKey].addData(payload.cpos_lookup, 'cpos_lookup');
  results[objectKey].addData(payload.text_lookup, 'text_lookup');
  results[objectKey].addData(results.metaData);
  results[objectKey].query = results.data.query;
  results[objectKey].corpus_type = type;
  results[objectKey].match_count += 1;
  results[objectKey].cpos_ranges = payload.cpos_ranges;
  results[objectKey].fullContext = true;
  console.info('Results data has been saved.', results);
  // Notify view to update progress bar
  client.notifyView('results-data-recieving', {type: type,
    progress: payload.progress})
  if (payload.progress === 100) {
    client.isBusy = false;
    if (objectKey === 'fullResultsData') {
      // Get match count per text from results.data only for fullResultsData
      results[objectKey].text_lookup = results.data.text_lookup;
    }
    client.notifyView('results-data-recieved', {type: type,
                                                results: results,
                                                fullContext: true});
  }
}

// export callbacks
export {
  prepareQueryData,
  saveMetaData,
  saveQueryData,
  getResultsData,
  saveResultsData,
};